@@ -1,19 +1,19 @@
+import logging
+import platform
 from ctypes import c_char
 from ctypes.util import find_library
-import logging
-from snap7.snap7exceptions import Snap7Exception
 
-import platform
+from snap7.exceptions import Snap7Exception
+
 if platform.system() == 'Windows':
-    from ctypes import windll as cdll
+    from ctypes import windll as cdll  # type: ignore
 else:
     from ctypes import cdll
 
-
 logger = logging.getLogger(__name__)
 
 # regexp for checking if an ipv4 address is valid.
-ipv4 = "^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$"
+ipv4 = r"^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$"
 
 
 class ADict(dict):
@@ -24,12 +24,13 @@ class ADict(dict):
     __setattr__ = dict.__setitem__
 
 
-class Snap7Library(object):
+class Snap7Library:
     """
     Snap7 loader and encapsulator. We make this a singleton to make
     sure the library is loaded only once.
     """
     _instance = None
+
     def __new__(cls, *args, **kwargs):
         if not cls._instance:
             cls._instance = object.__new__(cls)
@@ -47,25 +48,25 @@ class Snap7Library(object):
         self.cdll = cdll.LoadLibrary(self.lib_location)
 
 
-def load_library(lib_location=None):
+def load_library(lib_location: str = None):
     """
     :returns: a ctypes cdll object with the snap7 shared library loaded.
     """
     return Snap7Library(lib_location).cdll
 
 
-def check_error(code, context="client"):
+def check_error(code: int, context: str = "client"):
     """
     check if the error code is set. If so, a Python log message is generated
     and an error is raised.
     """
-    if code:
+    if code and code != 1:
         error = error_text(code, context)
         logger.error(error)
         raise Snap7Exception(error)
 
 
-def error_text(error, context="client"):
+def error_text(error, context: str = "client") -> bytes:
     """Returns a textual explanation of a given error number
 
     :param error: an error integer
@@ -73,7 +74,7 @@ def error_text(error, context="client"):
     :returns: the error string
     """
     assert context in ("client", "server", "partner")
-    logger.debug("error text for %s" % hex(error))
+    logger.debug(f"error text for {hex(error)}")
     len_ = 1024
     text_type = c_char * len_
     text = text_type()
