@@ -1,11 +1,11 @@
-import unittest
 import ctypes
 import logging
-import mock
+import unittest
+from unittest import mock
 
-import snap7.snap7types
 import snap7.error
 import snap7.server
+import snap7.types
 
 logging.basicConfig(level=logging.WARNING)
 
@@ -22,28 +22,28 @@ class TestServer(unittest.TestCase):
 
     def test_register_area(self):
         db1_type = ctypes.c_char * 1024
-        self.server.register_area(snap7.snap7types.srvAreaDB, 3, db1_type())
+        self.server.register_area(snap7.types.srvAreaDB, 3, db1_type())
 
     def test_error(self):
         for error in snap7.error.server_errors:
             snap7.common.error_text(error, context="client")
 
     def test_event(self):
-        event = snap7.snap7types.SrvEvent()
+        event = snap7.types.SrvEvent()
         self.server.event_text(event)
 
     def test_get_status(self):
         server, cpu, num_clients = self.server.get_status()
 
     def test_get_mask(self):
-        self.server.get_mask(snap7.snap7types.mkEvent)
-        self.server.get_mask(snap7.snap7types.mkLog)
+        self.server.get_mask(snap7.types.mkEvent)
+        self.server.get_mask(snap7.types.mkLog)
         # invalid kind
         self.assertRaises(Exception, self.server.get_mask, 3)
 
     def test_lock_area(self):
         from threading import Thread
-        area_code = snap7.snap7types.srvAreaDB
+        area_code = snap7.types.srvAreaDB
         index = 1
         db1_type = ctypes.c_char * 1024
         # we need to register first
@@ -70,10 +70,10 @@ class TestServer(unittest.TestCase):
         self.assertRaises(AssertionError, self.server.set_cpu_status, -1)
 
     def test_set_mask(self):
-        self.server.set_mask(kind=snap7.snap7types.mkEvent, mask=10)
+        self.server.set_mask(kind=snap7.types.mkEvent, mask=10)
 
     def test_unlock_area(self):
-        area_code = snap7.snap7types.srvAreaDB
+        area_code = snap7.types.srvAreaDB
         index = 1
         db1_type = ctypes.c_char * 1024
 
@@ -85,7 +85,7 @@ class TestServer(unittest.TestCase):
         self.server.unlock_area(area_code, index)
 
     def test_unregister_area(self):
-        area_code = snap7.snap7types.srvAreaDB
+        area_code = snap7.types.srvAreaDB
         index = 1
         db1_type = ctypes.c_char * 1024
         self.server.register_area(area_code, index, db1_type())
@@ -94,16 +94,18 @@ class TestServer(unittest.TestCase):
     def test_events_callback(self):
         def event_call_back(event):
             logging.debug(event)
+
         self.server.set_events_callback(event_call_back)
 
     def test_read_events_callback(self):
         def read_events_call_back(event):
             logging.debug(event)
+
         self.server.set_read_events_callback(read_events_call_back)
 
     def test_pick_event(self):
         event = self.server.pick_event()
-        self.assertEqual(type(event), snap7.snap7types.SrvEvent)
+        self.assertEqual(type(event), snap7.types.SrvEvent)
         event = self.server.pick_event()
         self.assertFalse(event)
 
@@ -117,25 +119,26 @@ class TestServer(unittest.TestCase):
 
     def test_get_param(self):
         # check the defaults
-        self.assertEqual(self.server.get_param(snap7.snap7types.LocalPort), 1102)
-        self.assertEqual(self.server.get_param(snap7.snap7types.WorkInterval), 100)
-        self.assertEqual(self.server.get_param(snap7.snap7types.MaxClients), 1024)
+        self.assertEqual(self.server.get_param(snap7.types.LocalPort), 1102)
+        self.assertEqual(self.server.get_param(snap7.types.WorkInterval), 100)
+        self.assertEqual(self.server.get_param(snap7.types.MaxClients), 1024)
 
         # invalid param for server
         self.assertRaises(Exception, self.server.get_param,
-                          snap7.snap7types.RemotePort)
-
+                          snap7.types.RemotePort)
 
 
 class TestServerBeforeStart(unittest.TestCase):
     """
     Tests for server before it is started
     """
+
     def setUp(self):
         self.server = snap7.server.Server()
 
     def test_set_param(self):
-        self.server.set_param(snap7.snap7types.LocalPort, 1102)
+        self.server.set_param(snap7.types.LocalPort, 1102)
+
 
 class TestLibraryIntegration(unittest.TestCase):
     def setUp(self):
