@@ -1,8 +1,7 @@
-import unittest
 import re
+import unittest
 
-from snap7 import util, snap7types
-
+from snap7 import util, types
 
 test_spec = """
 
@@ -20,21 +19,55 @@ test_spec = """
 13      testReal     REAL
 17      testDword    DWORD
 21      testint2     INT
+23      testDint     DINT
+27      testWord     WORD
+29      tests5time   S5TIME
+31      testdateandtime DATE_AND_TIME
+43      testusint0      USINT
+44      testsint0       SINT
 """
 
 _bytearray = bytearray([
-    0, 0,                                          # test int
+    0, 0,  # test int
     4, 4, ord('t'), ord('e'), ord('s'), ord('t'),  # test string
-    128*0 + 64*0 + 32*0 + 16*0 +
-    8*1 + 4*1 + 2*1 + 1*1,                         # test bools
+    128 * 0 + 64 * 0 + 32 * 0 + 16 * 0
+    + 8 * 1 + 4 * 1 + 2 * 1 + 1 * 1,                 # test bools
     68, 78, 211, 51,                               # test real
     255, 255, 255, 255,                            # test dword
     0, 0,                                          # test int 2
-    ])
+    128, 0, 0, 0,                                  # test dint
+    255, 255,                                      # test word
+    0, 16,                                         # test s5time, 0 is the time base,
+                                                   # 16 is value, those two integers should be declared together
+    32, 7, 18, 23, 50, 2, 133, 65,                 # these 8 values build the date and time 12 byte total
+                                                   # data typ together, for details under this link
+                                                   # https://support.industry.siemens.com/cs/document/36479/date_and_time-format-bei-s7-?dti=0&lc=de-DE
+    254, 254, 254, 254, 254, 127                   # test small int
+])
 
 
 class TestS7util(unittest.TestCase):
 
+    def test_get_s5time(self):
+        """
+        S5TIME extraction from bytearray
+        """
+        test_array = bytearray(_bytearray)
+
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+
+        self.assertEqual(row['tests5time'], '0:00:00.100000')
+
+    def test_get_dt(self):
+        """
+        DATE_AND_TIME extraction from bytearray
+        """
+        test_array = bytearray(_bytearray)
+
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+
+        self.assertEqual(row['testdateandtime'], '2020-07-12T17:32:02.854000')
+
     def test_get_string(self):
         """
         Text extraction from string from bytearray
@@ -65,17 +98,41 @@ class TestS7util(unittest.TestCase):
         y = row['testint2']
         self.assertEqual(x, 0)
         self.assertEqual(y, 0)
-        
+
     def test_set_int(self):
         test_array = bytearray(_bytearray)
         row = util.DB_Row(test_array, test_spec, layout_offset=4)
         row['ID'] = 259
         self.assertEqual(row['ID'], 259)
 
-    def test_set_int_roundtrip(self):
-        DB1 = (snap7types.wordlen_to_ctypes[snap7types.S7WLByte]*4)()
+    def test_get_usint(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        value = row.get_value(43, 'USINT')  # get value
+        self.assertEqual(value, 254)
 
-        for i in range(-(2**15)+1, (2**15)-1):
+    def test_set_usint(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        row['testusint0'] = 255
+        self.assertEqual(row['testusint0'], 255)
+
+    def test_get_sint(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        value = row.get_value(44, 'SINT')  # get value
+        self.assertEqual(value, 127)
+
+    def test_set_sint(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        row['testsint0'] = 127
+        self.assertEqual(row['testsint0'], 127)
+
+    def test_set_int_roundtrip(self):
+        DB1 = (types.wordlen_to_ctypes[types.S7WLByte] * 4)()
+
+        for i in range(-(2 ** 15) + 1, (2 ** 15) - 1):
             util.set_int(DB1, 0, i)
             result = util.get_int(DB1, 0)
             self.assertEqual(i, result)
@@ -84,18 +141,18 @@ class TestS7util(unittest.TestCase):
         test_array = bytearray(_bytearray)
         row = util.DB_Row(test_array, test_spec, layout_offset=4)
         for value in (
-                    -32768,
-                    -16385,
-                    -256,
-                    -128,
-                    -127,
-                    0,
-                    127,
-                    128,
-                    255,
-                    256,
-                    16384,
-                    32767):
+                -32768,
+                -16385,
+                -256,
+                -128,
+                -127,
+                0,
+                127,
+                128,
+                255,
+                256,
+                16384,
+                32767):
             row['ID'] = value
             self.assertEqual(row['ID'], value)
 
@@ -161,6 +218,32 @@ class TestS7util(unittest.TestCase):
         row = util.DB_Row(test_array, test_spec, layout_offset=4)
         self.assertEqual(row['testDword'], 4294967295)
 
+    def test_set_dint(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        # The range of numbers is -2147483648 to 2147483647 +
+        row.set_value(23, 'DINT', 2147483647)  # set value
+        self.assertEqual(row['testDint'], 2147483647)
+
+    def test_get_dint(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        value = row.get_value(23, 'DINT')  # get value
+        self.assertEqual(value, -2147483648)
+
+    def test_set_word(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        # The range of numbers is 0 to 65535
+        row.set_value(27, 'WORD', 0)  # set value
+        self.assertEqual(row['testWord'], 0)
+
+    def test_get_word(self):
+        test_array = bytearray(_bytearray)
+        row = util.DB_Row(test_array, test_spec, layout_offset=4)
+        value = row.get_value(27, 'WORD')  # get value
+        self.assertEqual(value, 65535)
+
     def test_export(self):
         test_array = bytearray(_bytearray)
         row = util.DB_Row(test_array, test_spec, layout_offset=4)
